from rest_framework import serializers
from pandas import DataFrame


if hasattr(serializers, 'ListSerializer'):
    # Django REST Framework 3
    BaseSerializer = serializers.ListSerializer
    USE_LIST_SERIALIZERS = True
else:
    # Django REST Framework 2
    BaseSerializer = serializers.Serializer
    USE_LIST_SERIALIZERS = False


class PandasSerializer(BaseSerializer):
    """
    Transforms dataset into a dataframe and applies an index
    """
    read_only = True
    index_none_value = None

    def get_index(self, dataframe):
        model_serializer = getattr(self, 'child', self)
        if getattr(model_serializer.Meta, 'model', None):
            return ['id']
        return None

    def get_dataframe(self, data):
        dataframe = DataFrame(data)
        index = self.get_index(dataframe)
        if index:
            if self.index_none_value is not None:
                for key in index:
                    dataframe[key].fillna(self.index_none_value, inplace=True)
            dataframe.set_index(index, inplace=True)
        else:
            # Name auto-index column to ensure valid CSV output
            dataframe.index.name = 'row'
        return dataframe

    def transform_dataframe(self, dataframe):
        view = self.context.get('view', None)
        if view and hasattr(view, 'transform_dataframe'):
            return self.context['view'].transform_dataframe(dataframe)
        return dataframe

    @property
    def data(self):
        data = super(PandasSerializer, self).data
        if data:
            dataframe = self.get_dataframe(data)
            return self.transform_dataframe(dataframe)
        else:
            return DataFrame([])


class SimpleSerializer(serializers.Serializer):
    """
    Simple serializer for non-model (simple) views
    """

    # DRF 3
    def to_representation(self, obj):
        return obj

    # DRF 2
    def to_native(self, obj):
        return obj



from rest_framework.views import APIView
from rest_framework.generics import ListAPIView
from rest_framework.mixins import ListModelMixin
from rest_framework.viewsets import GenericViewSet
from rest_framework.response import Response

from django.conf import settings
from rest_framework.settings import perform_import

from .serializers import (
    SimpleSerializer, PandasSerializer, USE_LIST_SERIALIZERS
)

PANDAS_RENDERERS = getattr(settings, "PANDAS_RENDERERS", None)
if PANDAS_RENDERERS is None:
    PANDAS_RENDERERS = (
        "rest_pandas.renderers.PandasCSVRenderer",
        "rest_pandas.renderers.PandasTextRenderer",
        "rest_pandas.renderers.PandasJSONRenderer",
        "rest_pandas.renderers.PandasExcelRenderer",
        "rest_pandas.renderers.PandasOldExcelRenderer",
        "rest_pandas.renderers.PandasPNGRenderer",
        "rest_pandas.renderers.PandasSVGRenderer",
    )

PANDAS_RENDERERS = perform_import(PANDAS_RENDERERS, "PANDAS_RENDERERS")


class PandasMixin(object):
    renderer_classes = PANDAS_RENDERERS
    paginate_by = None
    pandas_serializer_class = PandasSerializer

    def with_list_serializer(self, cls):
        if not USE_LIST_SERIALIZERS:
            # Django REST Framework 2 used the instance serializer for lists
            class SerializerWithListSerializer(
                    self.pandas_serializer_class, cls):
                pass
        else:

            # DRF3 uses a separate list_serializer_class; set if not present
            meta = getattr(cls, 'Meta', object)
            if getattr(meta, 'list_serializer_class', None):
                return cls

            class SerializerWithListSerializer(cls):
                class Meta(meta):
                    list_serializer_class = self.pandas_serializer_class

        return SerializerWithListSerializer


class PandasSimpleView(PandasMixin, APIView):
    """
    Simple (non-model) Pandas API view; override get_data
    with a function that returns a list of dicts.
    """
    serializer_class = SimpleSerializer

    def get_data(self, request, *args, **kwargs):
        return []

    def get(self, request, *args, **kwargs):
        data = self.get_data(request, *args, **kwargs)
        serializer_class = self.with_list_serializer(self.serializer_class)
        serializer = serializer_class(data, many=True)
        return Response(serializer.data)


class PandasView(PandasMixin, ListAPIView):
    """
    Pandas-capable model list view
    """
    def get_serializer_class(self, *args, **kwargs):
        cls = super(PandasView, self).get_serializer_class(*args, **kwargs)
        return self.with_list_serializer(cls)


class PandasViewSet(PandasMixin, ListModelMixin, GenericViewSet):
    """
    Pandas-capable model ViewSet (list only)
    """
    def get_serializer_class(self, *args, **kwargs):
        cls = super(PandasViewSet, self).get_serializer_class(*args, **kwargs)
        return self.with_list_serializer(cls)
